#include "keymap.h"
#include "lighting.h"
#include "pomodoro.h"

#define _TR _TRANS_LAYER
#define _TL _TALI_LAYER
#define _T2 _TALI_LAYER2
#define _T3 _TALI_LAYER3
#define _GL _GAME_LAYER
#define _FL _FUNCTION_LAYER
#define _F2 _FUNCTION2_LAYER
#define _UL _UNB_W_LAY
#define _U2 _UNB_W_LAY_2
#define _SA _SOLAR_ASH
#define _TT _TEAMS_LAYER
#define _PL _POM_LAYER
#define _P2 _POM_LAYER2

enum alt_keycodes {
    U_T_AUTO = SAFE_RANGE, //USB Extra Port Toggle Auto Detect / Always Active
    U_T_AGCR,              //USB Toggle Automatic GCR control
    DBG_TOG,               //DEBUG Toggle On / Off
    DBG_MTRX,              //DEBUG Toggle Matrix Prints
    DBG_KBD,               //DEBUG Toggle Keyboard Prints
    DBG_MOU,               //DEBUG Toggle Mouse Prints
    MD_BOOT,               //Restart into bootloader after hold timeout
    ALT_END
};

enum ashe_keycodes {
    AD_WIND = ALT_END, // Special layer behavior
    AD_UASH, // Æ: Upper-case ash
    AD_LASH, // æ: Lower-case ash
    AD_TEAM, // Teams push-to-talk macro
    AD_MOUS, // Toggle mouse wiggle
    AD_LOCK, // Lock screen and toggle layer

    // Pomodoro
    ADP_PAU, // Pause
    ADP_NEW, // New
    ADP_CCL, // Cancel
    ADP_DEL, // Delete all
    ADP_HID, // Show / Hide
};

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
    /*
    [DEFAULT] = LAYOUT(
        KC_ESC,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,     KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS, KC_EQL,  KC_BSPC, KC_DEL,
        KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,     KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_LBRC, KC_RBRC, KC_BSLS, KC_HOME,
        KC_CAPS, KC_A,    KC_S,    KC_D,    KC_F,    KC_G,     KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT,          KC_ENT,  KC_PGUP,
        KC_LSFT, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,     KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT,          KC_UP,   KC_PGDN,
        KC_LCTL, KC_LGUI, KC_LALT,                             KC_SPC,                             KC_RALT, MO(1),   KC_LEFT, KC_DOWN, KC_RGHT
    ),
    */
    [_TYPING_LAYER] = LAYOUT_65_ansi_blocker(
        KC_GESC, KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS, KC_EQL,  KC_BSPC, KC_RCTL, \
        KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_LBRC, KC_RBRC, KC_BSLS, KC_DEL, \
        MO(_F2), KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT,          KC_ENT,  KC_HOME, \
        KC_LSFT, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT,          KC_UP,   KC_END, \
        KC_LCTL, AD_WIND, KC_LALT,                            KC_SPC,                             KC_RALT, MO(_FL), KC_LEFT, KC_DOWN, KC_RGHT  \
    ),
    [_TRANS_LAYER] = LAYOUT_65_ansi_blocker(
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_TEAMS_LAYER] = LAYOUT_65_ansi_blocker(
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, AD_TEAM, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_GAME_LAYER] = LAYOUT_65_ansi_blocker(
        KC_ESC,  _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        XXXXXXX, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        KC_LSFT, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        KC_LCTL, XXXXXXX, KC_LALT,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_UNB_W_LAY] = LAYOUT_65_ansi_blocker(
        KC_ESC,  _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, KC_Q,    KC_E,    _______, _______, _______, _______, _______, _______, \
        XXXXXXX, KC_Q,    _______, KC_E,    _______, _______, KC_A,    KC_S,    KC_W,    KC_D,    _______, _______,          KC_ESC,  _______, \
        _______, _______, _______, _______, _______, _______, KC_SPC,  _______, _______, _______, _______, _______,          _______, _______, \
        _______, XXXXXXX, TG(_U2),                            KC_F,                               TG(_U2), MO(_FL), _______, _______, _______  \
    ),
    [_UNB_W_LAY_2] = LAYOUT_65_ansi_blocker(
        KC_ESC,  _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, _______, _______, _______, _______, _______, \
        XXXXXXX, KC_Q,    _______, KC_E,    _______, _______, KC_F,    KC_J,    KC_K,    KC_D,    _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, XXXXXXX, _______,                            _______,                            _______, XXXXXXX, _______, _______, _______  \
    ),
    [_SOLAR_ASH] = LAYOUT_65_ansi_blocker(
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, KC_F6, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, KC_F5, \
        KC_C,    _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_FUNCTION_LAYER] = LAYOUT_65_ansi_blocker(
        KC_GRV,  KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F11,  KC_F12,  RCTL(KC_BSPC), RALT(KC_PSCR), \
        _______, RGB_SPD, RGB_VAI, RGB_SPI, RGB_HUI, TG(_TL), _______, U_T_AUTO,U_T_AGCR,_______, KC_PSCR, KC_SLCK, KC_PAUS, _______, KC_DEL, \
        _______, RGB_RMOD,RGB_VAD, RGB_MOD, RGB_HUD, TG(_GL), TG(_UL), _______, _______, AD_LOCK, _______, _______,          RCTL(KC_ENT), KC_VOLU, \
        _______, RGB_TOG, _______, _______, _______, MD_BOOT, NK_TOGG, DBG_TOG, _______, _______, RCTL(KC_SLSH), _______,    KC_PGUP, KC_VOLD, \
        KC_NLCK, _______, _______,                      KC_MEDIA_PLAY_PAUSE,                      _______, MO(_FL), RCTL(KC_LEFT), KC_PGDN, RCTL(KC_RGHT)  \
    ),
    [_FUNCTION2_LAYER] = LAYOUT_65_ansi_blocker(
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, TG(_TR), _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        MO(_F2), _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        KC_CAPS, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            AD_MOUS,                            _______, _______, _______, _______, _______  \
    ),
    [_WIN_LAYER] = LAYOUT_65_ansi_blocker(
        KC_ESC,  _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, TG(_TT), _______, _______, _______, TG(_SA), _______, _______, _______, _______, _______, \
        _______, AD_UASH, AD_LASH, _______, _______, _______, KC_LEFT, KC_DOWN, KC_UP,   KC_RGHT, _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, OSL(_PL),                           _______,                            _______, _______, _______, _______, _______  \
    ),
    [_TALI_LAYER] = LAYOUT(
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        MO(_T2), XXXXXXX, XXXXXXX,                            XXXXXXX,                            XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX  \
    ),
    [_TALI_LAYER2] = LAYOUT(
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, MO(_T3), \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_TALI_LAYER3] = LAYOUT(
        TG(_TL), _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, \
        _______, _______, _______,                            _______,                            _______, _______, _______, _______, _______  \
    ),
    [_POM_LAYER] = LAYOUT_65_ansi_blocker(
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, OSL(_P2), ADP_CCL, ADP_HID, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX,                            ADP_NEW,                            XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX  \
    ),
    [_POM_LAYER2] = LAYOUT_65_ansi_blocker(
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, ADP_DEL, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX,          XXXXXXX, XXXXXXX, \
        XXXXXXX, XXXXXXX, XXXXXXX,                            XXXXXXX,                            XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX  \
    ),

};


// Runs just one time when the keyboard initializes.
void matrix_init_user(void) {
    // Enable or disable debugging
    debug_enable=false;
    debug_matrix=false;
    debug_keyboard=false;
    //rgb_enabled_flag = true;          // Initially, keyboard RGB is enabled. Change to false config.h initializes RGB disabled.
};

void keyboard_post_init_user(void) {
    rgb_matrix_enable();
}

#define MODS_SHIFT  (get_mods() & MOD_BIT(KC_LSHIFT) || get_mods() & MOD_BIT(KC_RSHIFT))
#define MODS_CTRL  (get_mods() & MOD_BIT(KC_LCTL) || get_mods() & MOD_BIT(KC_RCTRL))
#define MODS_ALT  (get_mods() & MOD_BIT(KC_LALT) || get_mods() & MOD_BIT(KC_RALT))

bool process_record_user(uint16_t keycode, keyrecord_t *record) {
    static uint32_t key_timer;
    static bool gui_disable_tap;
    bool had_shift = MODS_SHIFT;

    // Enable / disable WIN_LAYER
    if (keycode == AD_WIND) {
        if (record->event.pressed) {
            key_timer = timer_read32();
            gui_disable_tap = false;
            layer_on(_WIN_LAYER);
        } else {
            layer_off(_WIN_LAYER);

            if (!gui_disable_tap && timer_elapsed32(key_timer) < 500) {
                tap_code(KC_LGUI);
            }
        }

        return false;
    }

    // Special handling of transparent keys in win layer
    if (IS_LAYER_ON(_WIN_LAYER)) {
        gui_disable_tap = true;

        const keypos_t key = record->event.key;
        const uint16_t keymap_value = keymaps[_WIN_LAYER][key.row][key.col];

        if (keymap_value == KC_TRNS) {
            if(record->event.pressed) {
                register_code(KC_LGUI);
                tap_code(keycode);
                unregister_code(KC_LGUI);
            }
            return false;
        }
    }

    // Whenever not in Win layer, or win non-trans keys
    switch (keycode) {
        case KC_GESC:
            if (record->event.pressed && MODS_CTRL) {
                // Ctrl + ` hacks for VSCode
                if (!MODS_SHIFT) {
                    SEND_STRING("`");
                } else {
                // Still let ctrl + shift + esc work for task manager
                    register_code(KC_LCTL);
                    register_code(KC_LSHIFT);
                    tap_code(KC_ESC);
                    unregister_code(KC_LCTL);
                    unregister_code(KC_LSHIFT);
                }

                return false;
            }
            return true;

        case AD_UASH:
            if(record->event.pressed) {
                send_alt_code(198);
            }
            return false;

        case AD_LASH:
            if(record->event.pressed) {
                send_alt_code(230);
            }
            return false;

        // Toggle Teams mute
        case AD_TEAM:
            register_code(KC_LCTL);
            register_code(KC_LSHIFT);
            tap_code(KC_M);
            unregister_code(KC_LCTL);
            unregister_code(KC_LSHIFT);
            return false;

        case AD_MOUS:
            if (!record -> event.pressed) return false;
            enable_mouse_wiggle = !enable_mouse_wiggle;
            return false;

        case AD_LOCK:
            if (record -> event.pressed) {
                // Lock device
                if (had_shift) {
                    unregister_code(KC_RSHIFT);
                }

                register_code(KC_LGUI);
                tap_code(KC_L);
                unregister_code(KC_LGUI);

                if (had_shift) {
                    register_code(KC_RSHIFT);
                }
            } else {
                if (had_shift) {
                    layer_on(_TALI_LAYER);
                }
            }
            return false;

        // Pomodoro new, play / payse
        case ADP_NEW:
            if (!record->event.pressed) return false;
            new_pom();
            return false;

        // Pomodoro cancel
        case ADP_CCL:
            if (!record->event.pressed) return false;
            cancel_pom();
            return false;

        // Pomodoro delete
        case ADP_DEL:
            if (!record->event.pressed) return false;
            delete_pom();
            return false;

        // Show / hide
        case ADP_HID:
            if (!record->event.pressed) return false;
            show_pom();
            return false;

        case U_T_AUTO:
            if (record->event.pressed && MODS_SHIFT && MODS_CTRL) {
                TOGGLE_FLAG_AND_PRINT(usb_extra_manual, "USB extra port manual mode");
            }
            return false;
        case U_T_AGCR:
            if (record->event.pressed && MODS_SHIFT && MODS_CTRL) {
                TOGGLE_FLAG_AND_PRINT(usb_gcr_auto, "USB GCR auto mode");
            }
            return false;
        case DBG_TOG:
            if (record->event.pressed) {
                TOGGLE_FLAG_AND_PRINT(debug_enable, "Debug mode");
            }
            return false;
        case DBG_MTRX:
            if (record->event.pressed) {
                TOGGLE_FLAG_AND_PRINT(debug_matrix, "Debug matrix");
            }
            return false;
        case DBG_KBD:
            if (record->event.pressed) {
                TOGGLE_FLAG_AND_PRINT(debug_keyboard, "Debug keyboard");
            }
            return false;
        case DBG_MOU:
            if (record->event.pressed) {
                TOGGLE_FLAG_AND_PRINT(debug_mouse, "Debug mouse");
            }
            return false;
        case MD_BOOT:
            if (record->event.pressed) {
                key_timer = timer_read32();
            } else {
                if (timer_elapsed32(key_timer) >= 500) {
                    reset_keyboard();
                }
            }
            return false;
        case RGB_TOG:
            if (record->event.pressed) {
                switch (rgb_matrix_get_flags()) {
                    case LED_FLAG_ALL: {
                        rgb_matrix_set_flags(LED_FLAG_KEYLIGHT | LED_FLAG_MODIFIER);
                        rgb_matrix_set_color_all(0, 0, 0);
                    }
                    break;
                    case LED_FLAG_KEYLIGHT | LED_FLAG_MODIFIER: {
                        rgb_matrix_set_flags(LED_FLAG_UNDERGLOW);
                        rgb_matrix_set_color_all(0, 0, 0);
                    }
                    break;
                    case LED_FLAG_UNDERGLOW: {
                        rgb_matrix_set_flags(LED_FLAG_NONE);
                        rgb_matrix_disable_noeeprom();
                    }
                    break;
                    default: {
                        rgb_matrix_set_flags(LED_FLAG_ALL);
                        rgb_matrix_enable_noeeprom();
                    }
                    break;
                }
                return false;
            }
        default:
            break; //Process all other keycodes normally
    }

    return true;
}

void rgb_matrix_indicators_user(void) {
    static uint32_t wiggle_timer;
    static bool wiggle_left;

    if (disable_layer_color ||
        rgb_matrix_get_flags() == LED_FLAG_NONE ||
        rgb_matrix_get_flags() == LED_FLAG_UNDERGLOW) {
            return;
        }

    set_layer_color(get_highest_layer(layer_state));

    tick_pom();

    if (enable_mouse_wiggle) {
        hsv_matrix_set_color(61, (uint8_t[3]) C_TALI);

        if (!wiggle_timer) {
            wiggle_timer = timer_read32();
        }

        if (timer_elapsed32(wiggle_timer) > 1500) {
            wiggle_timer = 0;
            wiggle_left = !wiggle_left;

            if (wiggle_left) {
                tap_code(KC_MS_LEFT);
            } else {
                tap_code(KC_MS_RIGHT);
            }
        }
    }
}

const uint16_t NUMPAD_CODES[] = {
    KC_KP_0,
    KC_KP_1,
    KC_KP_2,
    KC_KP_3,
    KC_KP_4,
    KC_KP_5,
    KC_KP_6,
    KC_KP_7,
    KC_KP_8,
    KC_KP_9,
};

void send_alt_code(int code) {
    bool numlock_enabled = host_keyboard_led_state().num_lock;
    if (!numlock_enabled) {
        register_code(KC_NUMLOCK);
        unregister_code(KC_NUMLOCK);
    }

    short digit1 = code / 1000 % 10;
    short digit2 = code / 100  % 10;
    short digit3 = code / 10   % 10;
    short digit4 = code        % 10;

    register_code(KC_LALT);
    tap_code(NUMPAD_CODES[digit1]);
    tap_code(NUMPAD_CODES[digit2]);
    tap_code(NUMPAD_CODES[digit3]);
    tap_code(NUMPAD_CODES[digit4]);
    unregister_code(KC_LALT);

    if (!numlock_enabled) {
        register_code(KC_NUMLOCK);
        unregister_code(KC_NUMLOCK);
    }
}